//Cytoscape related requires !
window.oCanvas = global.oCanvas = require("ocanvas");
var cytoscape = window.cytoscape = require('cytoscape');
var panzoom = require('cytoscape-panzoom');
//var cxtmenu = require('cytoscape-cxtmenu');
var navigator = require('cytoscape-navigator');
var cyqtip = require('cytoscape-qtip');
var regCose = require('cytoscape-cose-bilkent');
var grid_guide = require('cytoscape-grid-guide');
var undoRedo = require('cytoscape-undo-redo');
var contextMenus = require('cytoscape-context-menus');
var nodeResize = require('cytoscape-node-resize');
var edgeEditing = require('cytoscape-edge-editing');
var konva = require('konva');
var viewUtilities = require('cytoscape-view-utilities');
require('bootstrap-select');


//Panzoom options
var panzoomOpts = require('./../Misc/PanzoomOptions.js');
var styleSheet = require('./../Misc/GraphStyleSheet.js');
var edgeHandleOpts = require('./../Misc/EdgeHandlesOptions.js');
var PromptConfirmationView = require('./../BackboneViews/PromptConfirmationView.js');
var LayoutProperties = require('./../BackboneViews/LayoutPropertiesView.js');
var GenomicDataExplorerView = require('./../BackboneViews/GenomicDataExplorerView.js');
var PathwayDetailsView = require('./../BackboneViews/PathwayDetailsView.js');
var GridOptionsView = require('./../BackboneViews/GridOptionsView.js');
var CBioPortalAccessView = require('./../BackboneViews/CbioPortalAccessView.js');

//Other requires
require('./../ViewHandlers/MenuBarHandlers.js');
require('./../ViewHandlers/ToolbarHandlers');
require('./../Utils/GraphUtilities.js');

var QtipManager = require('./QtipManager.js');
var ContextMenuManager = require('./ContextMenuManager.js');
var DragDropNodeAddPlugin = require('./../Utils/DragDropNodeAddPlugin.js');
var EditorActionsManager = require('./EditorActionsManager.js');
var GridOptionsManager = require('./GridOptionsManager.js');
var ViewOperationsManager = require('./ViewOperationsManager.js');
var FileOperationsManager = require('./FileOperationsManager.js');
var SaveLoadUtilities = require('./../Utils/SaveLoadUtility.js');
var CBioPortalAccessor = require('./../Utils/cBioPortalAccessor.js');

var notify = require('bootstrap-notify');
window.notificationManager = require('./../Utils/NotificationFactory');

module.exports = (function () {
    function AppManager(isCollaborative, shareDBManager, isCbioPortal) {
        this.isCollaborative = isCollaborative;
        this.shareDBManager = shareDBManager;
        this.isCbioPortal = isCbioPortal;
        this.init();
        this.createSampleMenu();
        this.createCBioPortalAccessModal();
    }

    AppManager.prototype.init = function () {
        //Initializes cytoscape
        this.initCyJS();
        //Initialize cytoscape based handlers here
        this.initCyHandlers();
        this.initKeyboardHandlers();
        this.initUndoRedoFunctionality();
        this.initCBioPortalFunctionalities();
        var that = this;
        //  window.onresize = function () {
        //      that.placePanzoomAndOverlay();
        //  }

        //Create portal accessor
        window.portalAccessor = new CBioPortalAccessor();
        window.appManager = this;
    };

    AppManager.prototype.placePanzoomAndOverlay = function () {
        //TODO place navigator !!!
        var offset = 5;
        var topCy = $('.cyContainer').offset().top;
        var bottomCy = $('.cyContainer').offset().bottom;
        var leftCy = $('.cyContainer').offset().left;
        var rightCy = $('.cyContainer').offset().right;
        var heightCy = $('.cyContainer').outerHeight();
        var widthCy = $('.cyContainer').outerWidth();
        var heightNavigator = $('.cytoscape-navigator-wrapper').outerHeight();
        var widthNavigator = $('.cytoscape-navigator-wrapper').outerWidth();
        var heightPatwayNavbar = $('.pathway-navbar').outerHeight();
        var heightPathwayToolbar = $('.pathway-toolbar').outerHeight();
        var widthSideBar = $('.sideBarWrapper').outerWidth();
        var widthcBioPortalSideBar = $('.cBioPortal-sidebar').outerWidth();


        var heightPanzoom = $('.cy-panzoom').outerHeight();
        var widthPanzoom = $('.cy-panzoom').outerWidth();

        if(!this.isCbioPortal) {
            $('.cytoscape-navigator-wrapper').css('top', heightCy + topCy - heightNavigator - offset);
            $('.cytoscape-navigator-wrapper').css('left', widthCy + leftCy - widthNavigator - offset);
        }
        else {
            $('.cytoscape-navigator-wrapper').css('top', heightCy + topCy - heightNavigator - heightPathwayToolbar - heightPatwayNavbar - 3 * offset);
            $('.cytoscape-navigator-wrapper').css('left', widthCy + leftCy - widthNavigator + widthcBioPortalSideBar + offset - widthSideBar);
        }

        //Relative is used so that its position depends on the below properties
        $('.cy-panzoom').css('position', 'relative');
        $('.cy-panzoom').css('top', 2);
        $('.cy-panzoom').css('left', widthCy - 57);
        $('.cy-panzoom').css('z-index', 1039);
        //Makes the width of panzoom container to 0
        $('.cy-panzoom').css('width', 0);
    }

    AppManager.prototype.createSampleMenu = function () {
        //Get template file data first
        var self = this;
        var request = new XMLHttpRequest();
        request.onreadystatechange = function () {
            if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
                var templateData = JSON.parse(request.responseText);

                for (var key in templateData) {
                    if (templateData.hasOwnProperty(key)) {
                        var newTCGAMenu = $('<li class="dropdown-submenu" id="' + key + '">' +
                            '<a href="#">' + key + '</a>' +
                            '</li>');
                        var newTCGAPathway = $('<ul class="dropdown-menu"></ul>');

                        for (var i in templateData[key]) {
                            var newPath = templateData[key][i];
                            var pName = newPath.replace(/-/gi, " ").substring(0, newPath.length - 4);
                            var sampleLink = $('<li><a  path="' + newPath + '" href="#">' + pName + '</a></li>');
                            sampleLink.on('click', checkMenuClickHandler);

                            //Add it to pan cancer menu
                            if (key.includes('PanCancer')) {
                                //panCancerSubMenu
                                $('#panCancerSubMenu').append(sampleLink);
                            }
                            else {
                                newTCGAPathway.append(sampleLink);
                                newTCGAMenu.append(newTCGAPathway);
                            }
                        }

                        //Add sub menus if they do not include pancaner and creighton
                        if (!key.includes('PanCancer') && !key.includes('Creighton')) {
                            $('#sampleSubMenu').append(newTCGAMenu);
                            if (self.isCbioPortal) {
                                $('#templateSubMenu').append(newTCGAMenu);
                            }
                        }

                    }
                }
            }
        };

        //Checks whether there is a visible pathway and displays a warning
        function checkMenuClickHandler(event) {
            if (cy.elements().length != 0)
                window.appManager.promptConfirmationView.render(function () {
                    sampleMenuClickHandler(event)
                });
            else
                sampleMenuClickHandler(event);
        };

        function updateTCGAInformation(title, desription) {
            document.getElementById("TCGA-header").innerHTML = title;
            document.getElementById("TCGA-textarea").innerText = desription;
        }

        function sampleMenuClickHandler(event) {
            var request = new XMLHttpRequest();
            request.onreadystatechange = function () {
                if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
                    var allEles = SaveLoadUtilities.parseGraph(request.responseText);
                    window.editorActionsManager.loadFile(allEles.nodes, allEles.edges);
                    window.undoRedoManager.reset();
                    window.appManager.pathwayDetailsView.updatePathwayProperties({
                        fileName: allEles.title + ".txt",
                        pathwayTitle: allEles.title,
                        pathwayDescription: allEles.description
                    });
                    if(self.isCbioPortal) {
                        updateTCGAInformation(allEles.title, allEles.description);
                    }
                }
            };
            //Send request for selected pathway
            var pathwayName = event.target.attributes[0].value;
            request.open("GET", "/pathway?filename=" + pathwayName);
            request.send();
        }


        request.open("GET", "/getTemplateFileData");
        request.send();
    };

    AppManager.prototype.getPathwayData = function () {
        return this.pathwayDetailsView.getPathwayData();
    };

    AppManager.prototype.createCBioPortalAccessModal = function () {
        var self = this;

        this.portalAccessView = new CBioPortalAccessView({
            el: $("#cbioPortalAccessDiv")
        });

        window.portalAccessor.fetchCancerStudies(function (cancerStudies) {
            self.portalAccessView.updateCancerStudies(cancerStudies);
        });
    };

    AppManager.prototype.initCyJS = function () {
        // panzoom(cytoscape, $);  // register extension
        //cxtmenu( cytoscape, $ ); // register extension
        // cyqtip(cytoscape, $); // register extension
        regCose(cytoscape); // register extension
        // navigator(cytoscape); // register extension
        // grid_guide(cytoscape, $); // register extension
        // undoRedo(cytoscape); // register extension
        // contextMenus(cytoscape, $); // register extension
        nodeResize(cytoscape, $, konva); //register extension
        // edgeEditing(cytoscape, $); // register extension
        // viewUtilities(cytoscape, $); // register extension


        window.edgeAddingMode = 0;
        // var allEles = SaveLoadUtilities.parseGraph(sampleGraph);
        cy = window.cy = cytoscape({
            container: document.querySelector('#cy'),
            boxSelectionEnabled: true,
            autounselectify: false,
            wheelSensitivity: 0.1,
            style: styleSheet,
            // elements: allEles,
            ready: function () {
            },
            textureOnViewport: false,
            motionBlur: true,
            layout: {name: 'preset'}
        });

        //Create Manager Classes
        window.editorActionsManager = this.editorActionsManager = new EditorActionsManager(this.isCollaborative,
            this.shareDBManager,
            window.cy,
            this.isCbioPortal);
        window.gridOptionsManager = new GridOptionsManager();
        window.viewOperationsManager = new ViewOperationsManager();
        window.fileOperationsManager = new FileOperationsManager();


        this.qtipManager = new QtipManager(window.cy);
        this.cxtMenuManager = new ContextMenuManager(window.cy, this.editorActionsManager);
        this.dragDropNodeAddManager = new DragDropNodeAddPlugin(this.editorActionsManager);

        //Render layout properties view after editor actions manager is created !
        this.layoutPropertiesView = new LayoutProperties({
            el: $('#layoutPropertiesDiv'),
            editorActionsManager: this.editorActionsManager
        }).render();

        this.promptConfirmationView = new PromptConfirmationView({
            el: $('#promptConfirmationDiv')
        }).render();

        this.gridOptionsView = new GridOptionsView({
            el: $('#gridOptionsDiv')
        }).render();

        // this.nodeResizeOptionsView = new NodeResizeOptionsView({
        //     el: $('#nodeResizeOptionsDiv')
        // }).render();

        this.genomicDataExplorerView = new GenomicDataExplorerView({
            el: $('#genomicDataExplorerDiv'),
            editorActionsManager: this.editorActionsManager
        }).render();

        this.pathwayDetailsView = new PathwayDetailsView({
            el: $('#pathwayDetailsDiv')
        }).render();

        //Initialize panzoom
        cy.panzoom(panzoomOpts);

        //Node Add initialization
        cy.nodeadd(
            {
                //Once the explanationText is cast to uppercase they will be node types
                components:
                    [
                        {
                            container: $('#simpleNodeDiv'),
                            nodeType: 'Gene',
                            // icon: 'fa fa-square-o'
                        },
                        {
                            container: $('#familyNodeDiv'),
                            nodeType: 'Family',
                            // icon: 'fa fa-square-o'
                        },
                        {
                            container: $('#complexNodeDiv'),
                            nodeType: 'Complex',
                            icon: 'fa fa-square-o'
                        },
                        {
                            container: $('#compartmentNodeDiv'),
                            nodeType: 'Compartment',
                            icon: 'fa fa-square-o'
                        },
                        {
                            container: $('#processNodeDiv'),
                            nodeType: 'Process',
                            // icon: 'fa fa-square-o'
                        }
                    ]

            });

        //Edge Handles initialization
        cy.edgehandles(edgeHandleOpts);

        //Navigator for cytoscape js
        var navDefaults = {
            container: '.cytoscape-navigator-wrapper' // can be a HTML or jQuery element or jQuery selector
            , viewLiveFramerate: 0 // set false to update graph pan only on drag end; set 0 to do it instantly; set a number (frames per second) to update not more than N times per second
            , thumbnailEventFramerate: 10 // max thumbnail's updates per second triggered by graph updates
            , thumbnailLiveFramerate: false // max thumbnail's updates per second. Set false to disable
            , dblClickDelay: 200 // milliseconds
            , removeCustomContainer: true // destroy the container specified by user on plugin destroy
            , rerenderDelay: 100 // ms to throttle rerender updates to the panzoom for performance
        };

        var nav = cy.navigator(navDefaults); // get navigator instance, nav

        var edgeEditingOptions = {
            // this function specifies the positions of bend points
            bendPositionsFunction: function (ele) {
                return ele.data('bendPointPositions');
            },
            // whether to initilize bend points on creation of this extension automatically
            initBendPointsAutomatically: true,
            // whether the bend editing operations are undoable (requires cytoscape-undo-redo.js)
            undoable: true,
            // the size of bend shape is obtained by multipling width of edge with this parameter
            bendShapeSizeFactor: 6,
            // whether to start the plugin in the enabled state
            enabled: true,
            // title of add bend point menu item (User may need to adjust width of menu items according to length of this option)
            addBendMenuItemTitle: "Add Bend Point",
            // title of remove bend point menu item (User may need to adjust width of menu items according to length of this option)
            removeBendMenuItemTitle: "Remove Bend Point",

            handleReconnectEdge: this.editorActionsManager.reconnectEdge.bind(this.editorActionsManager),
        };
        window.edgeEditing = cy.edgeEditing(edgeEditingOptions);

        cy.nodeResize({
            padding: 5, // spacing between node and grapples/rectangle
            undoable: true, // and if cy.undoRedo exists

            grappleSize: 8, // size of square dots
            grappleColor: "#ffc90e", // color of grapples
            inactiveGrappleStroke: "inside 1px blue",
            boundingRectangle: true, // enable/disable bounding rectangle
            boundingRectangleLineDash: [4, 8], // line dash of bounding rectangle
            boundingRectangleLineColor: "ffc90e",
            boundingRectangleLineWidth: 1.5,
            zIndex: 999,

            moveSelectedNodesOnKeyEvents: function () {
                return true;
            },

            minWidth: function (node) {
                var data = node.data("resizeMinWidth");
                return data ? data : 15;
            }, // a function returns min width of node
            minHeight: function (node) {
                var data = node.data("resizeMinHeight");
                return data ? data : 15;
            }, // a function returns min height of node

            // Getters for some style properties the defaults returns ele.css('property-name')
            // you are encouraged to override these getters
            getCompoundMinWidth: function (node) {
                return node.style('min-width');
            },
            getCompoundMinHeight: function (node) {
                return node.style('min-height');
            },
            getCompoundMinWidthBiasRight: function (node) {
                return node.style('min-width-bias-right');
            },
            getCompoundMinWidthBiasLeft: function (node) {
                return node.style('min-width-bias-left');
            },
            getCompoundMinHeightBiasTop: function (node) {
                return node.style('min-height-bias-top');
            },
            getCompoundMinHeightBiasBottom: function (node) {
                return node.style('min-height-bias-bottom');
            },


            isFixedAspectRatioResizeMode: function (node) {
                return node.is(".fixedAspectRatioResizeMode")
            },// with only 4 active grapples (at corners)
            isNoResizeMode: function (node) {
                return undefined;
            }, // no active grapples

            // These optional function will be executed to set the width/height of a node in this extension
            // Using node.css() is not a recommended way (http://js.cytoscape.org/#eles.style) to do this. Therefore, overriding these defaults
            // so that a data field or something like that will be used to set node dimentions instead of directly calling node.css()
            // is highly recommended (Of course this will require a proper setting in the stylesheet).
            setWidth: function (node, width) {
                node.style('width', width);
            },
            setHeight: function (node, height) {
                node.style('height', height);
            },
            setCompoundMinWidth: function (node, minWidth) {
                node.style('min-width', minWidth);
            },
            setCompoundMinHeight: function (node, minHeight) {
                node.style('min-height', minHeight);
            },
            setCompoundMinWidthBiasLeft: function (node, minWidthBiasLeft) {
                node.style('min-width-bias-left', minWidthBiasLeft);
            },
            setCompoundMinWidthBiasRight: function (node, minHeightBiasRight) {
                node.style('min-width-bias-right', minHeightBiasRight);
            },
            setCompoundMinHeightBiasTop: function (node, minHeightBiasTop) {
                node.style('min-height-bias-top', minHeightBiasTop);
            },
            setCompoundMinHeightBiasBottom: function (node, minHeightBiasBottom) {
                node.style('min-height-bias-bottom', minHeightBiasBottom);
            },

            cursors: { // See http://www.w3schools.com/cssref/tryit.asp?filename=trycss_cursor
                // May take any "cursor" css property
                default: "default", // to be set after resizing finished or mouseleave
                inactive: "not-allowed",
                nw: "nw-resize",
                n: "n-resize",
                ne: "ne-resize",
                e: "e-resize",
                se: "se-resize",
                s: "s-resize",
                sw: "sw-resize",
                w: "w-resize"
            },

            resizeToContentCueImage: '/assets/nodes/ResizeCue.svg',

            resizeToContentFunction: this.editorActionsManager.resizeNodesToContent.bind(this.editorActionsManager),

        });

        var viewUtilitiesOpts = {
            node: {
                highlighted: {
                    'border-width': 2,
                    'border-color': '#bc1142'
                }, // styles for when nodes are highlighted.
                unhighlighted: {
                    'opacity': function (ele) {
                        // We return the same opacity because to override the unhibhlighted ele opacity in view-utilities
                        return ele.css('opacity');
                    }
                }// styles for when nodes are unhighlighted.}
            },
            edge: {
                highlighted: {}, // styles for when edges are highlighted.
                unhighlighted: {
                    'opacity': function (ele) {
                        // We return the same opacity because to override the unhibhlighted ele opacity in view-utilities
                        return ele.css('opacity');
                    }
                } // styles for when edges are unhighlighted.
            },
            setVisibilityOnHide: false, // whether to set visibility on hide/show
            setDisplayOnHide: true, // whether to set display on hide/show
            neighbor: function (node) { // return desired neighbors of tapheld node
                return false;
            },
            neighborSelectTime: 500 //ms, time to taphold to select desired neighbors
        }

        window.viewUtilities = cy.viewUtilities(viewUtilitiesOpts);

        this.placePanzoomAndOverlay();
    };

    AppManager.prototype.initCyHandlers = function () {
        var that = this;

        var tappedBefore;
        var tappedTimeout;
        cy.on('tap', function (event) {
            var tappedNow = event.target;
            if (tappedTimeout && tappedBefore) {
                clearTimeout(tappedTimeout);
            }
            if (tappedBefore === tappedNow) {
                tappedNow.trigger('doubleTap');
                tappedBefore = null;
            } else {
                tappedTimeout = setTimeout(function () {
                    tappedBefore = null;
                }, 300);
                tappedBefore = tappedNow;
            }
        });

        cy.on('doubleTap', 'node', function (e) {
            var eventIsDirect = (e.target === this);

            if (eventIsDirect) {
                //Remove qtips
                $(".qtip").remove();
                that.qtipManager.addQtipToElements(e.target);
                var api = this.qtip('api');
                if (api) {
                    api.show();
                }
            }
        });

        cy.on('doubleTap', 'edge', function (e) {
            var eventIsDirect = (e.target === this);

            if (eventIsDirect) {
                $(".qtip").remove();
                that.qtipManager.addQtipToElements(e.target);
                // var api = this.qtip('api');
                // if (api) {
                //     api.show();
                // }
            }
        });

        cy.on('select', 'node', function (e) {
            window.editorActionsManager.pushSelectedNodeStack(e.target);
        });

        // cy.on('select', 'edge', function( e )
        // {
        //   var eventIsDirect = (e.target === this);
        //   $(".qtip").remove();
        //
        //   if( eventIsDirect ) {
        //       that.qtipManager.addQtipToElements(e.target);
        //   }
        // });

        cy.on('unselect', 'node', function (e) {
            window.editorActionsManager.removeElementFromSelectedNodeStack(e.target);
        });

        cy.on('free', 'node', function (e) {
            //Collect all nodes with descendants in case of compounds
            var selectedNodes = cy.nodes(':selected');
            var nodes = e.target;
            nodes = nodes.union(nodes.descendants());
            nodes = nodes.union(selectedNodes);
            that.editorActionsManager.moveElements(nodes);
        });

        cy.on('layoutstop', function (event) {
            that.editorActionsManager.postLayout();
        });

        // //TODO fix this when cytoscape is updated !!!
        // //Due to cytoscape.js bug, only workaround that worked :(
        // cy.on('add', 'node', function(event)
        // {
        //     // event.target.select();
        //     cy.style().update();
        //     cy.forceRender();
        // });

        cy.on("noderesize.resizeend", function (e, type, node) {
            //Updates 'data' properties from 'style'
            node.data('w', node.width());
            node.data('h', node.height());
            //Used for collaborative mode
            that.editorActionsManager.resizeElements(node);
        });

        cy.on('bendPointMovement', function () {
            editorActionsManager.updateEdgeBendPoints(cy.elements(":selected").first());
        });

        cy.on('noderesize.moveend', function () {
            editorActionsManager.changeNodePositionsByArrows(cy.nodes(":selected"));
        });

    };

    AppManager.prototype.initKeyboardHandlers = function () {
        if (!this.isCollaborative && !this.isCbioPortal) {
            $(document).keydown(function (e) {
                if (e.which === 89 && (e.ctrlKey || event.metaKey)) {
                    window.undoRedoManager.redo();
                }
                else if (e.which === 90 && (e.ctrlKey || event.metaKey)) {
                    window.undoRedoManager.undo();
                }
            });
        }
        else {
            $('a[role="redo"]').hide();
            $('a[role="undo"]').hide();
        }
        $(document).keydown(function (e) {
            if (e.which === 65 && (e.ctrlKey || event.metaKey)) {
                var tn = document.activeElement.tagName;
                if (tn != "TEXTAREA" && tn != "INPUT") {
                    e.preventDefault();
                    cy.elements().select();
                }
            }
            else if (e.which === 8 || e.which === 46) {
                var tn = document.activeElement.tagName;
                if (tn != "TEXTAREA" && tn != "INPUT") {
                    var selectedElements = cy.$(':selected');
                    editorActionsManager.removeElement(selectedElements);
                }
            }
        });
    };

    AppManager.prototype.initUndoRedoFunctionality = function () {
        if (this.isCollaborative || this.isCbioPortal) {
            $('[role="undo"]').hide();
            $('[role="redo"]').hide();
            document.getElementById("localOrCollaborativeToolbar").style.display = "none";
        }
    };


    AppManager.prototype.initCBioPortalFunctionalities = function () {
        if (this.isCbioPortal) {
            contextMenu = cy.contextMenus('get');

            //Destroy context menu
            contextMenu.destroy();
            //Hide toolbar, sidebar, navbar
            document.getElementById("pathway-toolbar").style.display = "none";
            document.getElementById("pathway-sidebar").style.display = "none";
            document.getElementById("pathway-navbar").style.display = "none";
            document.getElementById("cBioPortal-Wrapper").style.display = "inline-block";
            document.getElementById("about-model-header").innerHTML = "PathwayMapper Viewer 2.0";
        }
    };


    return AppManager;
})();
